@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2007 Nominum, Inc.
+# Copyright (C) 2003-2007, 2009-2011 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -13,11 +13,16 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 # OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
+import struct
+
 import dns.exception
 import dns.rdata
 import dns.tokenizer
+from dns._compat import text_type
+
 
 class HINFO(dns.rdata.Rdata):
+
     """HINFO record
 
     @ivar cpu: the CPU type
@@ -27,57 +32,54 @@ class HINFO(dns.rdata.Rdata):
     @see: RFC 1035"""
 
     __slots__ = ['cpu', 'os']
-    
+
     def __init__(self, rdclass, rdtype, cpu, os):
         super(HINFO, self).__init__(rdclass, rdtype)
-        self.cpu = cpu
-        self.os = os
+        if isinstance(cpu, text_type):
+            self.cpu = cpu.encode()
+        else:
+            self.cpu = cpu
+        if isinstance(os, text_type):
+            self.os = os.encode()
+        else:
+            self.os = os
 
     def to_text(self, origin=None, relativize=True, **kw):
         return '"%s" "%s"' % (dns.rdata._escapify(self.cpu),
                               dns.rdata._escapify(self.os))
-        
-    def from_text(cls, rdclass, rdtype, tok, origin = None, relativize = True):
+
+    @classmethod
+    def from_text(cls, rdclass, rdtype, tok, origin=None, relativize=True):
         cpu = tok.get_string()
         os = tok.get_string()
         tok.get_eol()
         return cls(rdclass, rdtype, cpu, os)
-    
-    from_text = classmethod(from_text)
 
-    def to_wire(self, file, compress = None, origin = None):
+    def to_wire(self, file, compress=None, origin=None):
         l = len(self.cpu)
         assert l < 256
-        byte = chr(l)
-        file.write(byte)
+        file.write(struct.pack('!B', l))
         file.write(self.cpu)
         l = len(self.os)
         assert l < 256
-        byte = chr(l)
-        file.write(byte)
+        file.write(struct.pack('!B', l))
         file.write(self.os)
-        
-    def from_wire(cls, rdclass, rdtype, wire, current, rdlen, origin = None):
-        l = ord(wire[current])
+
+    @classmethod
+    def from_wire(cls, rdclass, rdtype, wire, current, rdlen, origin=None):
+        l = wire[current]
         current += 1
         rdlen -= 1
         if l > rdlen:
             raise dns.exception.FormError
-        cpu = wire[current : current + l]
+        cpu = wire[current:current + l].unwrap()
         current += l
         rdlen -= l
-        l = ord(wire[current])
+        l = wire[current]
         current += 1
         rdlen -= 1
         if l != rdlen:
             raise dns.exception.FormError
-        os = wire[current : current + l]
+        os = wire[current: current + l].unwrap()
         return cls(rdclass, rdtype, cpu, os)
 
-    from_wire = classmethod(from_wire)
-
-    def _cmp(self, other):
-        v = cmp(self.cpu, other.cpu)
-        if v == 0:
-            v = cmp(self.os, other.os)
-        return v
